use chrono::NaiveDateTime;
use serde::{Deserialize, Serialize};

use crate::database::{models::search_result::ROWS_PER_PAGE, DatabaseHandler};

use super::search_result::SearchResult;

#[derive(Debug, Clone, sqlx::FromRow, Serialize, Deserialize)]
pub struct Teamrace {
    pub players: Vec<String>,
    pub map: String,
    pub time: f32,
    pub timestamp: NaiveDateTime,
}

impl Teamrace {
    /// This gets all the teamraces from the database a given player has partaken in.
    /// It is paginated into a SearchResult<Teamrace>!
    /// FIXME: Returns empty array for non-existent player.
    pub async fn get_teamraces_by_player(
        db: &DatabaseHandler,
        player: &str,
        page: i32,
    ) -> Result<SearchResult<Teamrace>, sqlx::Error> {
        let results = sqlx::query_file_as!(
            Teamrace,
            "sql/teamraces/get_teamraces_by_player.sql",
            player,
            page,
            ROWS_PER_PAGE
        )
        .fetch_all(&db.pool)
        .await?;

        let total_pages = sqlx::query!(
            "SELECT ceil(count(*) / $2) + 1 as total_pages FROM record_teamrace_array WHERE $1 = ANY(players)",
            player,
            ROWS_PER_PAGE as i64
        )
        .fetch_one(&db.pool)
        .await?
        .total_pages
        .map_or_else(|| 0, |as_i64| as_i64 as i32);

        Ok(SearchResult {
            results,
            total_pages,
            current_page: page,
        })
    }

    /// This gets all the teamraces from the database on a specific map.
    /// It is paginated into a SearchResult<Teamrace>!
    /// FIXME: Returns empty array for non-existent map.
    pub async fn get_teamraces_by_map(
        db: &DatabaseHandler,
        map: &str,
        page: i32,
    ) -> Result<SearchResult<Teamrace>, sqlx::Error> {
        let results = sqlx::query_file_as!(
            Teamrace,
            "sql/teamraces/get_teamraces_by_map.sql",
            map,
            page,
            ROWS_PER_PAGE
        )
        .fetch_all(&db.pool)
        .await?;

        let total_pages = sqlx::query!(
            "SELECT ceil(count(*) / $2) + 1 as total_pages FROM record_teamrace_array WHERE map = $1",
            map,
            ROWS_PER_PAGE as i64
        )
        .fetch_one(&db.pool)
        .await?
        .total_pages
        .map_or_else(|| 0, |as_i64| as_i64 as i32);

        Ok(SearchResult {
            results,
            total_pages,
            current_page: page,
        })
    }
}
